Package: rmsb
Title: Bayesian Regression Modeling Strategies
Version: 1.1-2
Date: 2025-04-13
Authors@R: 
    c(person(given = "Frank",
           family = "Harrell",
           role = c("aut", "cre"),
           email = "fh@fharrell.com",
           comment = c(ORCID = "0000-0002-8271-5493")),
			person(given = "Ben",
			       family = "Goodrich",
						 role = "ctb",
						 email = "benjamin.goodrich@columbia.edu",
						 comment = "contributed Stan code"),
			person(given = "Ben",
			       family = "Bolker",
						 role = "ctb",
						 comment = "wrote original code that is folded into the pdensityContour function"),
			person(given = "Doug",
			       family = "Bates",
						 role = "ctb",
						 comment = "write original code for highest posterior density interval that is folded into the HPDint function"))
Description: A Bayesian companion to the 'rms' package, 'rmsb' provides Bayesian model fitting, post-fit estimation, and graphics.  It implements Bayesian regression models whose fit objects can be processed by 'rms' functions such as 'contrast()', 'summary()', 'Predict()', 'nomogram()', and 'latex()'.  The fitting function currently implemented in the package is 'blrm()' for Bayesian logistic binary and ordinal regression with optional clustering, censoring, and departures from the proportional odds assumption using the partial proportional odds model of Peterson and Harrell (1990) <https://www.jstor.org/stable/2347760>.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://hbiostat.org/R/rmsb/
RoxygenNote: 7.3.2
Biarch: true
Depends: R (>= 3.4.0), rms (>= 8.0-0)
Imports: methods, Rcpp (>= 0.12.0), rstan (>= 2.26.23), Hmisc (>=
        5.2-3), survival (>= 3.1-12), ggplot2, MASS, cluster, digest,
        knitr, loo
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
Suggests: cmdstanr, bayesplot, mice
Additional_repositories: https://mc-stan.org/r-packages/
SystemRequirements: GNU make
NeedsCompilation: yes
Packaged: 2025-04-13 16:03:34 UTC; harrelfe
Author: Frank Harrell [aut, cre] (<https://orcid.org/0000-0002-8271-5493>),
  Ben Goodrich [ctb] (contributed Stan code),
  Ben Bolker [ctb] (wrote original code that is folded into the
    pdensityContour function),
  Doug Bates [ctb] (write original code for highest posterior density
    interval that is folded into the HPDint function)
Maintainer: Frank Harrell <fh@fharrell.com>
Repository: CRAN
Date/Publication: 2025-04-13 16:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 02:24:10 UTC; unix
Archs: rmsb.so.dSYM
