# Generated by roxygen2: do not edit by hand

S3method(ExProb,blrm)
S3method(Mean,blrm)
S3method(Quantile,blrm)
S3method(coef,rmsb)
S3method(plot,PostF)
S3method(plot,rmsb)
S3method(predict,blrm)
S3method(print,blrm)
S3method(print,blrmStats)
S3method(print,predict.blrm)
S3method(print,rmsb)
S3method(vcov,rmsb)
export(HPDint)
export(PostF)
export(blrm)
export(blrmStats)
export(cluster)
export(compareBmods)
export(distSym)
export(getParamCoef)
export(pdensityContour)
export(selectedQr)
export(stackMI)
export(stanDx)
export(stanDxplot)
export(stanGet)
export(tauFetch)
import(Hmisc)
import(Rcpp)
import(ggplot2)
import(methods)
import(rms)
importFrom(grDevices,contourLines)
importFrom(grDevices,dev.off)
importFrom(grDevices,gray)
importFrom(grDevices,png)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(rstan,optimizing)
importFrom(rstan,sampling)
importFrom(stats,approx)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,lowess)
importFrom(stats,median)
importFrom(stats,model.extract)
importFrom(stats,model.matrix)
importFrom(stats,plogis)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,capture.output)
useDynLib(rmsb, .registration = TRUE)
